<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\ApuracaoEfetivacao\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\CMP\Modules\Fornecimento\Modules\ApuracaoEfetivacao\Services\EfetivacaoService;

class EfetivacaoController extends \Teknisa\GER\Controllers\TeknisaCrud 
{
    private $mensagensRetorno = array('MENSAGENS' => [], 'INCONSISTENCIAS' => [], 'NRSEQAPUR' => []);
    
    public function __construct($crudService, ParameterBag $parameterBag, $connection, EfetivacaoService $service, $dataSourceManager, $fechamentoAF, $inconsistenciaAF) 
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->connection = $connection;
        $this->service = $service;
        $this->dataSourceManager = $dataSourceManager;
        $this->fechamentoAF = $fechamentoAF;
        $this->inconsistenciaAF = $inconsistenciaAF;
    }

    public function efetivar(DTO\Request $request, DTO\Response $response)
    {
        ini_set('max_execution_time', 9999);
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $params = array();
            $params['P_CDCENTCMP']    = $row['P_CDCENTCMP'];
            $params['P_NRSEQAPUR']    = $row['P_NRSEQAPUR'];
            $params['P_CDFILIOPELOG'] = $row['P_CDFILIOPELOG'];
            
            $adqCotacao = $this->adqCotacaoFull($params);
            
            $qtdeCotacoesApuradas = 0;
            foreach ($adqCotacao as $cotacao) {
                if($cotacao["IDITMSELAPUR"] == "S") {
                    $qtdeCotacoesApuradas++;
                }
            }
            
            if($qtdeCotacoesApuradas == 0) {
                throw new \Exception("Não há dados para efetivação de AF(s).");
            }
            
            $params = array();
            $params['P_NRSEQAPUR']   = $row['P_NRSEQAPUR'];
            $params['P_CDCENTCMP']   = $row['P_CDCENTCMP'];
            $params['P_CDFILLANC']   = $row['P_CDFILIOPELOG'];
            $params['P_CDOPERALANC'] = $row['P_CDOPERALANC'];
            $params['P_CDCOMPRADOR'] = $row['P_CDCOMPRADOR'];
            $adqEfetivaAF            = $this->getVd_EfetivaAF($params);
            
            if(empty($adqEfetivaAF)) {
                $params = array();
                $params['P_CDOPERADOR']  = $row['P_CDCOMPRADOR'];
                $params['P_DTAUTOFORN']  = $row['P_DTAUTOFORN'];
                $params['P_CDCENTCMP']   = $row['P_CDCENTCMP'];
                $params['P_NRSEQAPUR']   = $row['P_NRSEQAPUR'];
                $params['P_CDFILLANC']   = $row['P_CDFILIOPELOG'];
                $this->GeraAF($params);
            } else {
                throw new \Exception("Operação bloqueada. Não é possível efetivar a mesma apuração mais de uma vez.");
            }
            
            $this->service->commit();
            
            $response->addDataSet(new DataSet('data', [ $this->mensagensRetorno ]));
        } catch(\Exception $e) {
            
            $this->service->rollback();
            
            $this->excluiAF($row);
            
            $code = $e->getCode();
            
            switch ($code) {
                case 10:
                    $message = "AF não fechada. Ocorreu um erro ao tentar excluir conta a pagar relacionada.";
                    break;
                    
                case 20:
                    $message = "AF fechada. Não foi possível gerar conta a pagar.";
                    break;
            }
            
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function GeraAF($params) 
    {
        $this->service->executeEFETIVAR_AF($params);
        
        $eafParams['P_CDFILCENTCMP'] = $params['P_CDCENTCMP'];
        $eafParams = $this->service->adqEmpenhaAf($eafParams)[0];
        
        if($eafParams['IDUTEMPENHAF'] == 'S') {
            $empafParams['P_NRSEQAPUR']    = $params['P_NRSEQAPUR']; 
            $empafParams['P_CDFILLANC']    = $params['P_CDFILLANC']; 
            $empafParams['P_CDFILCENTCMP'] = $params['P_CDCENTCMP']; 
            $empafParams['P_CDOPERADOR']   = $params['P_CDOPERADOR'];
            $this->service->executeEMPENHAR_AF($empafParams);
        }
        
        $autfParams['P_CDFILLANC']   = $params['P_CDFILLANC'];
        $autfParams['P_NRSEQAPURAF'] = $params['P_NRSEQAPUR']; 
        $adqAutoForne = $this->service->adqAutoForne($autfParams);
        
        /*
            Retorna o número da apuração.
         */
        $this->setNrApuracao($autfParams['P_NRSEQAPURAF']);
        if(!empty($adqAutoForne)) {
            foreach($adqAutoForne as $autoForn) {
                $gravaInconsistencias = true;
                
                $inconsistencias = $this->inconsistenciaAF->verificaInconsistencias($autoForn['CDFILDEST'], $autoForn['CDFILLANC'], $autoForn['NRAUTOFORN'], $params['P_CDCENTCMP'], $gravaInconsistencias);
                
                if (self::hasInconsistencias($inconsistencias)) {
                    $bloqAF = $this->service->adqBloqAF(array("P_CDFILDEST" => $autoForn['CDFILDEST']))[0];
                    
                    $this->setInconsistenciasOnMensagensRetorno($inconsistencias);
                    /**
                     * TODO: Posteriormente, seria interessante implementar uma forma de abortar apenas
                     * o processo da AF que é inconsistente, e não o processo inteiro, permitindo assim
                     * ao usuário corrigir os erros e efetivar tudo, ou efetivar o que não houve erro.
                     */
                    if ($bloqAF['IDBLOQUEAAF'] == 'S') {
                        throw new \Exception('Não foi possível efetivar, pois existe(m) AF(s) inconsistente(s) ou fora do limite.', 1);
                    } else {
                        $this->setMensagemOnMensagensRetorno("Existe(m) AF(s) inconsistente(s) ou fora do limite. Para visualizá-las verifique o relatório \"AF's Inconsistentes/Fora do Limite\" na tela \"Análise de Autorização de Fornecimento\".");
                    }
                }
            }
            
            foreach($adqAutoForne as $autoForn) {
                //$this->fechaAF($autoForn);
                $this->fechamentoAF->fechaAF($autoForn['CDFILDEST'], $autoForn['CDFILLANC'], $autoForn['NRAUTOFORN'], $params['P_CDCENTCMP']);
                // $this->fechamentoAF->geraContaPagarOrcFinan($autoForn['P_CDFILDEST'], $autoForn['P_CDFILLANC'], $autoForn['P_NRAUTOFORN']);
            }
            
            // {Atualizando o preço da última AF na PRODFILI}
            $custoParams['P_CDFILDEST']   = null;   
            $custoParams['P_CDFILLANC']   = $params['P_CDFILLANC'];    
            $custoParams['P_NRAUTOFORN']  = null;  
            $custoParams['P_NRSEQAPURAF'] = $params['P_NRSEQAPUR'];
            $this->service->executeCUSTO_AF($custoParams);
        }
    }
    
    public function adqCotacaoFull($row) {
        $params = array();
        $params['P_CDCENTCMP'] = $row['P_CDCENTCMP'];
        $params['P_NRSEQAPUR'] = $row['P_NRSEQAPUR'];
        
        $nrlogcorteol = $this->service->adqNrlogcorteol($params);
        
        if (empty($nrlogcorteol)) {
            $params['P_CDPRODESTO'] = 'T';
            return $this->service->selectCotacaoWithParamNull($params);
        } else {
            $params['P_NRLOGCORTEOL'] = $nrlogcorteol;
            $params['P_CDPRODESTO'] = 'T';
            return $this->service->selectCotacaoWithParamNotNull($params);
        }
    }
    
    public function volta_QtdeDisponivel($params) {
        $adqCot = $this->service->adqCot_ItemAutF($params);
        
        foreach($adqCot as $value) {
            $params = array();
            $params['P_CDFILIAL']    = $value['CDFILIAL'];
            $params['P_NRCOLEPRECO'] = $value['NRCOLEPRECO'];
            $params['P_CDMARCA']     = $value['CDMARCA'];
            $params['P_CDREGIAO']    = $value['CDREGIAO'];
            $params['P_CDFORNECED']  = $value['CDFORNECED'];
            $params['P_CDPRODUTO']   = $value['CDPRODUTO'];
            
            $adqPedCol = $this->service->adqPedCol($params);
            if(!empty($adqPedCol) || !empty($adqPedCol[0]['NRPEDIDOCOL'])) {
                $params = array();
                $params['P_CDFILIAL']    = $adqPedCol[0]['CDFILIAL'];
                $params['P_NRPEDIDOCOL'] = $adqPedCol[0]['NRPEDIDOCOL'];
                $params['P_CDPRODUTO']   = $value['CDPRODUTO'];
                $params['P_CDMARCA']     = $value['CDMARCA'];
                $params['P_QTDEAF']      = $value['QTPROGENTAF'];
                
                $this->service->acmUpdPrMarcPedCol($params);
            }
        }
    }
    
    public function getVd_EfetivaAF($params) {
        $return = $this->service->adqEfetivaAF($params);
        
        if(empty($return)) {
            $this->service->acmInsEfetivaAF($params);
        }
        
        return $return;
    }
    
    private function fechaAF($item) {
        $paramsFechaAF['P_CDFILDEST']   = $item['CDFILDEST'];
        $paramsFechaAF['P_CDFILLANC']   = $item['CDFILLANC'];
        $paramsFechaAF['P_NRAUTOFORN']  = $item['NRAUTOFORN'];
        $this->service->proceFechamentoAF($paramsFechaAF);
    }
    
    private function excluiAF($row) {
        $paramsAdqInconsAF['P_CDFILLANC']   = $row['P_CDFILIOPELOG'];
        $paramsAdqInconsAF['P_NRAPURPRECO'] = $row['P_NRSEQAPUR'];
        $inconsAF =  $this->service->adqInconsAF($paramsAdqInconsAF);
        
        foreach ($inconsAF as $item) {
            $paramsVoltaQtde['P_CDFILDEST']   = $item['CDFILDEST'];
            $paramsVoltaQtde['P_CDFILLANC']   = $item['CDFILLANC'];
            $paramsVoltaQtde['P_NRAUTOFORN']  = $item['NRAUTOFORN'];
            
            $this->volta_QtdeDisponivel($paramsVoltaQtde);
        }
        
        $paramsAcmExcluiAF['P_CDFILLANC']   = $row['P_CDFILIOPELOG'];
        $paramsAcmExcluiAF['P_NRSEQAPURAF'] = $row['P_NRSEQAPUR'];
        $this->service->acmExcluiAF($paramsAcmExcluiAF);
    }
    
    public function hasInconsistencias($inconsistencias) {
        foreach ($inconsistencias as $nameIncons => $incons) {
            /**
             * TODO: Caso o valor 'N' não seja necessário na Análise de AF, remover
             * essa verificação do AF_FORA_LIM.
             */
            if ($nameIncons == 'AF_FORA_LIM' && $incons == 'N') {
                continue;
            }
            
            if (!empty($incons)) {
                return true;
            }
        }
        
        return false;
    }
    
    private function setInconsistenciasOnMensagensRetorno($inconsistencias) {
        foreach ($inconsistencias as $nameIncons => $incons) {
            /**
             * TODO: Analisar se é necessário incluir as inconsistências por AF.
             */
            $this->mensagensRetorno['INCONSISTENCIAS'][$nameIncons] = $incons;
        }
    }
    
    private function setMensagemOnMensagensRetorno($mensagem) {
        if (!in_array($mensagem, $this->mensagensRetorno['MENSAGENS'])) {
            array_push($this->mensagensRetorno['MENSAGENS'], $mensagem, true);
        }
    }
    
    private function setNrApuracao($nrapuracao) {
        array_push($this->mensagensRetorno['NRSEQAPUR'], $nrapuracao, true);
    }
}